// Работаем с Канвой
var Canvas =
{
	Width : 597,
	Height: 842,

	GetHtmlElement : function()
	{
		return document.getElementById("canvas");
	},

	Drawing_Init : function()
	{
		var Context = this.GetHtmlElement().getContext("2d");

		Context.fillStyle = "rgb(0,0,0)";
		Context.strokeRect(0, 0, this.Width, this.Height);
	},

	Clear : function()
	{
		var Context = this.GetHtmlElement().getContext("2d");

		Context.clearRect (0, 0, this.Width, this.Height );
		Context.strokeRect(0, 0, this.Width, this.Height);
	},

	BeginPath : function()
	{
		var Context = this.GetHtmlElement().getContext("2d");
		Context.beginPath();
	},

	MoveTo : function(X,Y)
	{
		var Context = this.GetHtmlElement().getContext("2d");
		Context.moveTo( X, Y );
	},

	LineTo : function(X,Y)
	{
		var Context = this.GetHtmlElement().getContext("2d");
		Context.lineTo( X, Y );
	},

	ClosePath : function()
	{
		var Context = this.GetHtmlElement().getContext("2d");
		Context.closePath();
	},

	Stroke : function()
	{
		var Context = this.GetHtmlElement().getContext("2d");
		Context.stroke();
	},

	SetLineWidth : function(W)
	{
		var Context = this.GetHtmlElement().getContext("2d");
		Context.lineWidth = W;
	},

	FillText : function(X,Y,Text)
	{
		var Context = this.GetHtmlElement().getContext("2d");
		Context.fillText( Text, X, Y );
	},

	Measure : function(Text)
	{
		var Context = this.GetHtmlElement().getContext("2d");
		var Width = Context.measureText( Text ).width;
		var Height = 0;

		return { Width : Width, Height : Height };
	},

	GetFont : function()
	{
		return this.GetHtmlElement().getContext("2d").font;
	},

	SetFont : function(font)
	{
		if ( "string" == typeof(font) )
			this.GetHtmlElement().getContext("2d").font = font;
		else if ( "object" == typeof(font) )
		{
			var sFont = "";
			if ( true === font.Bold )
				sFont += "bold ";
			if ( true === font.Italic )
				sFont += "italic ";

			sFont += font.FontSize + "pt" + " " + font.FontFamily;
			this.GetHtmlElement().getContext("2d").font = sFont;
		}
	},

	SetFillStyle : function(style)
	{
		this.GetHtmlElement().getContext("2d").fillStyle = style;
	},
    Focus : function()
    {
        this.GetHtmlElement().focus();
    }
};

// Работаем с внешим интфейсом
var Interface =
{
	TextPr :
	{
		Bold      : Default_TextPr_Bold,
		Italic    : Default_TextPr_Italic,
		Underline : Default_TextPr_Underline,
		Strikeout : Default_TextPr_Strikeout,
		FontSize  : Default_TextPr_FontSize,
		FontFamily: Default_TextPr_FontFamily
	},

    FlyTextPr :
    {
        Div : null,
        X   : 0,
        Y   : 0,
        W   : 0,
        H   : 0,
        Show: false // показали ли мы полностью уже (контроль первого появления)
    },

	UpdateTextPr : function(TextPr)
	{
		if ( "undefined" != typeof(TextPr) )
		{
			for ( var Item in TextPr )
			{
				switch( Item )
				{
					case "Bold"      : this.TextPr.Bold       = TextPr[Item];break;
					case "Italic"    : this.TextPr.Italic     = TextPr[Item];break;
					case "Underline" : this.TextPr.Underline  = TextPr[Item];break;
					case "Strikeout" : this.TextPr.Strikeout  = TextPr[Item];break;
					case "FontSize"  : this.TextPr.FontSize   = TextPr[Item];break;
					case "FontFamily": this.TextPr.FontFamily = TextPr[Item];break;
				}
			}
		}

		this.Internal_UpdateBold();
		this.Internal_UpdateItalic();
		this.Internal_UpdateUnderline();
		this.Internal_UpdateStrikeout();
		this.Internal_UpdateFontSize();
		this.Internal_UpdateFontFamily();
	},

	Internal_UpdateBold : function()
	{
		if ( this.TextPr.Bold )
			document.getElementById("Text_Bold").src = "Images/bold_clicked.png";
		else
			document.getElementById("Text_Bold").src = "Images/bold.png";

        if ( null != this.FlyTextPr.Div )
        {
            if ( this.TextPr.Bold )
                document.getElementById("FlyTextPr_Text_Bold").src = "Images/bold_clicked.png";
            else
                document.getElementById("FlyTextPr_Text_Bold").src = "Images/bold.png";
        }
	},
	Internal_UpdateItalic : function()
	{
		if ( this.TextPr.Italic )
			document.getElementById("Text_Italic").src = "Images/italic_clicked.png";
		else
			document.getElementById("Text_Italic").src = "Images/italic.png";

        if ( null != this.FlyTextPr.Div )
        {
            if ( this.TextPr.Italic )
                document.getElementById("FlyTextPr_Text_Italic").src = "Images/italic_clicked.png";
            else
                document.getElementById("FlyTextPr_Text_Italic").src = "Images/italic.png";
        }
	},
	Internal_UpdateUnderline : function()
	{
		if ( this.TextPr.Underline )
			document.getElementById("Text_Underline").src = "Images/underline_clicked.png";
		else
			document.getElementById("Text_Underline").src = "Images/underline.png";

        if ( null != this.FlyTextPr.Div )
        {
            if ( this.TextPr.Underline )
                document.getElementById("FlyTextPr_Text_Underline").src = "Images/underline_clicked.png";
            else
                document.getElementById("FlyTextPr_Text_Underline").src = "Images/underline.png";
        }
	},
	Internal_UpdateStrikeout : function()
	{
		if ( this.TextPr.Strikeout )
			document.getElementById("Text_Strikeout").src = "Images/strikeout_clicked.png";
		else
			document.getElementById("Text_Strikeout").src = "Images/strikeout.png";

        if ( null != this.FlyTextPr.Div )
        {
            if ( this.TextPr.Strikeout )
                document.getElementById("FlyTextPr_Text_Strikeout").src = "Images/strikeout_clicked.png";
            else
                document.getElementById("FlyTextPr_Text_Strikeout").src = "Images/strikeout.png";
        }
	},
	Internal_UpdateFontSize : function()
	{
		document.getElementById("Input_FontSize").value = this.TextPr.FontSize;

        if ( null != this.FlyTextPr.Div )
        {
            document.getElementById("FlyTextPr_Input_FontSize").value = this.TextPr.FontSize;
        }
	},
	Internal_UpdateFontFamily : function()
	{
		document.getElementById("Input_FontFamily").value = this.TextPr.FontFamily;

        if ( null != this.FlyTextPr.Div )
        {
            document.getElementById("FlyTextPr_Input_FontFamily").value = this.TextPr.FontFamily;
        }
	},

    FlyTextPr_Init : function(X,Y)
    {
        // Если окно уже есть, то удалим его
        this.FlyTextPr_Remove();

        var DivW = 172 + 2 + 4; // width  + border + padding
        var DivH = 54  + 2 + 4; // height + border + padding
        var DivY = Y - 30 - DivH;
        var DivX = X            ;

        var Div = document.createElement("div");

        Div.id = "FlyTextPr";
        Div.style.position = "absolute";
        Div.style.left     = X     + 50; // + 50 - TODO: сдвиг канвы, надо будет переделать понормальному
        Div.style.top      = DivY  + 70; // + 70
        Div.style.width    = 172;
        Div.style.height   = 54;
        Div.style.border   = "1px solid silver";
        Div.style.padding  = 2;
        Div.style.backgroundColor = "#fff";
        Div.style.zIndex   = 100;

        Div.innerHTML = "\<input id=\"FlyTextPr_Input_FontFamily\" style=\"float:left;width:130;height:30;margin-right:2px;margin-bottom: 2px\" type=\"text\" value=\"Arial\" onchange=\"Actions_SetFontFamily( this.value );\"/>\
                          <input id=\"FlyTextPr_Input_FontSize\" style=\"float:left;width:40;height:30;margin-bottom: 2px\" type=\"text\" value=\"10\" onChange=\"Actions_SetFontSize( this.value );\"/>\
                          <img id=\"FlyTextPr_Text_Bold\" style=\"float:left;\" src=\"Images/bold.png\" onclick=\"Actions_TextPr('Bold')\"/>\
                          <img id=\"FlyTextPr_Text_Italic\" style=\"float:left;\" src=\"Images/italic.png\" onclick=\"Actions_TextPr('Italic')\"/>\
                          <img id=\"FlyTextPr_Text_Underline\" style=\"float:left;\" src=\"Images/underline.png\" onclick=\"Actions_TextPr('Underline')\"/>\
                          <img id=\"FlyTextPr_Text_Strikeout\" style=\"float:left;\" src=\"Images/strikeout.png\" onclick=\"Actions_TextPr('Strikeout')\"/>";

        document.body.appendChild( Div );

        this.FlyTextPr.Div  = Div;
        this.FlyTextPr.X    = X;
        this.FlyTextPr.Y    = DivY;
        this.FlyTextPr.W    = DivW;
        this.FlyTextPr.H    = DivH;
        this.FlyTextPr.Show = false;
        this.UpdateTextPr();
    },
    FlyTextPr_Remove : function()
    {
        try
        {
            if ( null != this.FlyTextPr.Div )
            {
                document.body.removeChild( this.FlyTextPr.Div );

                this.FlyTextPr.Div  = null;
                this.FlyTextPr.X    = 0;
                this.FlyTextPr.Y    = 0;
                this.FlyTextPr.W    = 0;
                this.FlyTextPr.H    = 0;
                this.FlyTextPr.Show = false;
            }
        }
        catch(e)
        {

        }
    },
    FlyTextPr_SetOpacity : function(X,Y)
    {
        if ( null != this.FlyTextPr.Div )
        {
            // Вычисляем расстояние до BBox в зависимости от позиции, относительно BBox
            //
            //    1      2       3
            //        _______
            //    4  |   5   |   6
            //       |_______|
            //    7      8       9

            var Diff = -1;

            if ( X < this.FlyTextPr.X )
            {
                if ( Y < this.FlyTextPr.Y )
                    Diff = Math.max( this.FlyTextPr.Y - Y, this.FlyTextPr.X - X );
                else if ( Y > this.FlyTextPr.Y + this.FlyTextPr.H )
                    Diff = Math.max( Y - this.FlyTextPr.Y - this.FlyTextPr.H, this.FlyTextPr.X - X );
                else
                    Diff = this.FlyTextPr.X - X;
            }
            else if ( X > this.FlyTextPr.X + this.FlyTextPr.W )
            {
                if ( Y < this.FlyTextPr.Y )
                    Diff = Math.max( this.FlyTextPr.Y - Y, X - this.FlyTextPr.X - this.FlyTextPr.W );
                else if ( Y > this.FlyTextPr.Y + this.FlyTextPr.H )
                    Diff = Math.max( Y - this.FlyTextPr.Y - this.FlyTextPr.H, X - this.FlyTextPr.X - this.FlyTextPr.W );
                else
                    Diff = X - this.FlyTextPr.X - this.FlyTextPr.W;
            }
            else
            {
                if ( Y < this.FlyTextPr.Y )
                    Diff = this.FlyTextPr.Y - Y;
                else if ( Y > this.FlyTextPr.Y + this.FlyTextPr.H )
                    Diff = Y - this.FlyTextPr.Y - this.FlyTextPr.H;
                else
                    Diff = 0;
            }


            var Div = this.FlyTextPr.Div;
            if ( Diff > 300 )
            {
                // Удаляем элемент
                this.FlyTextPr_Remove();
            }
            else if ( Diff > 0 )
            {
                if ( true == this.FlyTextPr.Show )
                    Div.style.opacity = Math.max( (100 - Diff ) / 100.0, 0 );
                else
                    Div.style.opacity = Math.max( (35 - Diff ) / 35.0, 0 );
            }
            else
            {
                Div.style.opacity = 1;
                this.FlyTextPr.Show = true;
            }
        }
    }

};
// Работаем с "целью"

function Drawing_Target_Draw()
{
	if ( "block" != document.getElementById("Target").style.display )
		document.getElementById("Target").style.display = "block";
	else
		document.getElementById("Target").style.display = "none";
}

var Target =
{
	IntervalId : null,
	Size : 10,

	GetHtmlElement : function()
	{
		return document.getElementById("Target");
	},

	Update : function(X, Y)
	{
		var Targ = this.GetHtmlElement();
		Targ.style.left = X + 50;
		Targ.style.top  = Y + 70 - this.Size;
	},

	SetSize : function(newSize)
	{
		var Targ = this.GetHtmlElement();

		this.Size = newSize;
		Targ.style.height = (Number(this.Size) + Number(0.3 * this.Size));
	},

	// Добавляем мигающий курсор
	Drawing_Init : function()
	{
		if ( this.IntervalId )
			clearInterval( this.IntervalId );
		this.IntervalId = setInterval( "Drawing_Target_Draw()", 500 );
	},

	// Убираем мигающий курсор
	Drawing_Remove : function()
	{
		clearInterval( this.IntervalId );
		this.GetHtmlElement().style.display = "none";
	}
};
