// Содержимое параграфа должно иметь:
//
// 1. Type    - тип
// 2. Draw    - рисуем на контексте
// 3. Measure - измеряем
//---- после использования Measure -----
// 4. Width   - ширина
// 5. Height  - высота

var para_Unknown         =     -1; //
var para_Text            = 0x0001; // Текст
var para_Space           = 0x0002; // Пробелы
var para_TextPr          = 0x0003; // Свойства текста
var para_End             = 0x0004; // Конец параграфа
var para_NewLine         = 0x0010; // Новая строка
var para_NewLineRendered = 0x0011; // Рассчитанный перенос строки

var Default_TextPr_Bold       = false;
var Default_TextPr_Italic     = false;
var Default_TextPr_Underline  = false;
var Default_TextPr_Strikeout  = false;
var Default_TextPr_FontFamily = "Times New Roman";
var Default_TextPr_FontSize   = 14; // 14pt

var para_Cur         = 100;   // Текущая позиция курсора в параграфе
var para_Letter      = 101;  // буква
var para_NewLineCalc = 102;

// Класс ParaText
function ParaText(value)
{
	this.Value = value;
	this.Type  = para_Text;
}

ParaText.prototype.Draw = function(X, Y, Context)
{
	try
	{
		Context.FillText( X, Y, this.Value );
	}
	catch(e)
	{
		
	}
};

ParaText.prototype.Measure = function(Context)
{
	try
	{
		var Temp = Context.Measure( this.Value );
		this.Width  = Temp.Width;
		this.Height = Temp.Height;
	}
	catch(e)
	{
		this.Width  = 0;
		this.Height = 0;
	}
	return { Width : this.Width, Height : this.Height };
};

// Класс ParaSpace
function ParaSpace(Count)
{
	if ( "number" != typeof(Count) )
		this.Value = 1;
	else
		this.Value = Count;
	
	this.Type = para_Space;
}

ParaSpace.prototype.Draw = function(X,Y, Context)
{
	var String = "";
	if ( 1 == this.Value )
		String = " ";
	else
	{
		for ( var Index = 0; Index < this.Value; Index++ )
		{
			String += " ";
		}
	}
	
	try
	{
		Context.FillText( X, Y, String );
	}
	catch(e)
	{
		
	}
};

ParaSpace.prototype.Measure = function(Context)
{
	var String = "";
	if ( 1 == this.Value )
		String = " ";
	else
	{
		for ( var Index = 0; Index < this.Value; Index++ )
		{
			String += " ";
		}
	}
	
	try
	{
		var Temp = Context.Measure( String );
		this.Width  = Temp.Width;
		this.Height = Temp.Height;
	}
	catch(e)
	{
		this.Width  = 0;
		this.Height = 0;
	}
	
	return { Width : this.Width, Height : this.Height };
};

// Класс ParaTextPr
function ParaTextPr(Props)
{
	this.Type  = para_TextPr;
	this.Value = new Object();
	
	if ( "object" != typeof(Props) )
		return;
	
	for(var Prop in Props)
	{
		switch(Prop)
		{
			case "Bold"      : this.Value.Bold       = Props[Prop]; break;
			case "Italic"    : this.Value.Italic     = Props[Prop]; break;
			case "Underline" : this.Value.Underline  = Props[Prop]; break;
			case "Strikeout" : this.Value.Strikeout  = Props[Prop]; break;
			case "FontFamily": this.Value.FontFamily = Props[Prop]; break;
			case "FontSize"  : this.Value.FontSize   = Props[Prop]; break;
		}
	}
}

ParaTextPr.prototype.Draw = function()//(X,Y,Context)
{
	// Ничего не делаем
};

ParaTextPr.prototype.Measure = function()//(Context)
{
	this.Width  = 0;
	this.Height = 0;

	return { Width : this.Width, Height : this.Height };
};

// Класс окончание параграфа ParaEnd
function ParaEnd()
{
	this.Type = para_End;
}

ParaEnd.prototype.Draw = function(X,Y,Context)
{
	Context.FillText( X, Y, String.fromCharCode( 0x00B6 ) ); 
};

ParaEnd.prototype.Measure = function()//(Context)
{
	this.Width  = 0;
	this.Height = 0;

	return { Width : this.Width, Height : this.Height };
};

// Класс ParaNewLine
function ParaNewLine()
{
	this.Type = para_NewLine;
}

ParaNewLine.prototype.Draw = function(X,Y,Context)
{
	var oldFont = Context.GetFont();
	Context.SetFont( "11pt Arial" );
	Context.FillText( X, Y, String.fromCharCode( 0x21B2 ) );
	Context.SetFont( oldFont );
};

ParaNewLine.prototype.Measure = function()//(Context)
{
	this.Width  = 0;
	this.Height = 0;

	return { Width : this.Width, Height : this.Height };
};

// Класс ParaNewLineRendered
function ParaNewLineRendered()
{
	this.Type = para_NewLineRendered;
}

ParaNewLineRendered.prototype.Draw = function()//(X,Y,Context)
{
	// Ничего не делаем
};

ParaNewLineRendered.prototype.Measure = function()//(Context)
{
	this.Width  = 0;
	this.Height = 0;

	return { Width : this.Width, Height : this.Height };
};


